@@ -84,13 +84,22 @@
 #'   runApp(app)
 #' }
 #' @export
-runApp <- function(appDir=getwd(),
-                   port=getOption('shiny.port'),
-                   launch.browser = getOption('shiny.launch.browser', interactive()),
-                   host=getOption('shiny.host', '127.0.0.1'),
-                   workerId="", quiet=FALSE,
-                   display.mode=c("auto", "normal", "showcase"),
-                   test.mode=getOption('shiny.testmode', FALSE)) {
+runApp <- function(
+  appDir=getwd(),
+  port=getOption('shiny.port'),
+  launch.browser = getOption('shiny.launch.browser', interactive()),
+  host=getOption('shiny.host', '127.0.0.1'),
+  workerId="", quiet=FALSE,
+  display.mode=c("auto", "normal", "showcase"),
+  test.mode=getOption('shiny.testmode', FALSE)
+) {
+
+  # * Wrap **all** execution of the app inside the otel promise domain
+  # * While this could be done at a lower level, it allows for _anything_ within
+  #   shiny's control to allow for the opportunity to have otel active spans be
+  #   reactivated upon promise domain restoration
+  promises::local_otel_promise_domain()
+
   on.exit({
     handlerManager$clear()
   }, add = TRUE)
@@ -445,7 +454,9 @@ stopApp <- function(returnValue = invisible()) {
 #' @param host The IPv4 address that the application should listen on. Defaults
 #'   to the `shiny.host` option, if set, or `"127.0.0.1"` if not.
 #' @param display.mode The mode in which to display the example. Defaults to
-#'   `showcase`, but may be set to `normal` to see the example without
+#'   `"auto"`, which uses the value of `DisplayMode` in the example's
+#'   `DESCRIPTION` file. Set to `"showcase"` to show the app code and
+#'   description with the running app, or `"normal"` to see the example without
 #'   code or commentary.
 #' @param package The package in which to find the example (defaults to
 #'   `"shiny"`).
