@@ -1,7 +1,5 @@
-library(shiny)
-library(testthat)
+skip_if_not_installed("future")
 library(future, warn.conflicts = FALSE)
-library(promises)
 
 test_that("handles observers", {
   server <- function(input, output, session) {
@@ -746,7 +744,7 @@ test_that("promise chains evaluate in correct order", {
 
   server <- function(input, output, session) {
     r1 <- reactive({
-      promise(function(resolve, reject) {
+      promises::promise(function(resolve, reject) {
         pushMessage("promise 1")
         resolve(input$go)
       })$then(function(value) {
@@ -755,7 +753,7 @@ test_that("promise chains evaluate in correct order", {
       })
     })
     r2 <- reactive({
-      promise(function(resolve, reject) {
+      promises::promise(function(resolve, reject) {
         pushMessage("promise 2")
         resolve(input$go)
       })$then(function(value) {
